<?php

/**
 * 【MongoDB PHP类】
 *
 * 作者：陈叨叨 WWW.C-DD.CN
 * 日期：2019.04.24
 * 版本：1.0.0
 *
 * 在此基础上开发，表示感谢 https://blog.csdn.net/why444216978/article/details/85329366
 *
 * 基于PHP新版Mongo驱动开发，因为旧版驱动只支持到PHP5.X.X。
 * 基于Mongo4.0.9测试，因为Mongo自身原因，某些地方旧版本不兼容。
 * 由于\MongoDB\Driver\Manager驱动特性，此类初始化时，并不执行I/O，之后根据需要初始化连接。（因此可以在程序一开始就初始化此对象备用）
 * 尽量细致查询了每个基础类的参数及含义，认为有用的功能已经都写进来了。
 * 针对分布集群的功能全部忽略了，觉得如果需求大到需要上集群的话，直接买阿里云算了。
 * 除特殊需要，所有返回值都转化为了数组格式（翻译成人话）。
 * 条件参数可以使用正则，语法和PHP一样。比较符号也可以直接用人话，内部会自动处理。
 *
 * 话说Mongo的命令和返回都太不像人话了，一层层的嵌套跟JS一样恶心。
 */

namespace DB\Mongo;

class MongoDB{

    private $_manager;
    private $_host;
    private $_port;
    private $_username;
    private $_password;
    private $_db;

    /**
     * @var array 记录最后一条错误信息
     */
    public  $err = array("code" => null, "err" => null);

    /**
     * 【初始化连接】
     *
     * 函数内设置了默认超时时间。
     * 如果有其他附加参数可以通过$param一起传入，但是会经过过滤。
     *
     * @param  array $param     参数数组（host|port|username|password|db|...）
     * @return bool             成功返回true，失败返回false
     */
    public function __construct($param = array()){
        if (empty($param['host']) || empty($param['db'])) {
            $this->err = array("code" => 999, "msg" => "host or db is null");
            return false;
        }
        $this->_host = $param['host'];
        $this->_port = $param['port'] ? $param['port'] : '27017';
        $this->_username = $param['username'] ? urlencode($param['username']) : '';
        $this->_password = $param['password'] ? urlencode($param['password']) : '';
        $this->_db = $param['db'];
        if (!$this->_username) {
            $mongo = "mongodb://{$this->_host}:{$this->_port}/";
        } else {
            $mongo = "mongodb://{$this->_username}:{$this->_password}@{$this->_host}:{$this->_port}/";
        }
        $opt = array("connectTimeoutMS" => 1000, "socketTimeoutMS" =>3000, "wtimeoutMS" => 1000);
        foreach (array("connectTimeoutMS", "socketTimeoutMS", "wtimeoutMS", "ssl", "journal", "w")as $k) {
            if ($param[$k]) $opt[$k] = $param[$k];
        }
        try {
            $this->_manager = new \MongoDB\Driver\Manager($mongo, $opt);
            $this->err = array("code" => null, "msg" => null);
            return true;
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }
    public function getInstense() {
        return $this->_manager;
    }
    public function getDB() {
        return $this->_db;
    }
    public function getErr() {
        return $this->err;
    }
    /**
     * 【取得\Driver\BulkWrite对象】
     *
     * @param  bool $passDocumentValidation      跳过文档验证（默认false）
     * @param  bool $order                       按顺序执行（默认true）
     * @return bool|\MongoDB\Driver\BulkWrite    成功返回对象，失败返回false
     */
    public function getBulk($passDocumentValidation = null, $order = null) {
        $opt = array();
        if ($passDocumentValidation !== null) $opt["bypassDocumentValidation"] = (bool) $passDocumentValidation;
        if ($order                  !== null) $opt["ordered"]                  = (bool) $order;
        try {
            $this->err = array("code" => null, "msg" => null);
            return new \MongoDB\Driver\BulkWrite($opt);
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }
    /**
     * 【取得\Driver\WriteConcern对象】
     *
     * @param  int|string $wait                     等待方式(w:1/0/majority)（默认值majority）
     * @param  int        $time                     等待时间(ms)（此处默认值null，等于类初始化时的默认值）
     * @param  bool       $safe                     等待写入journal（默认值null，根据系统设置）
     * @return bool|\MongoDB\Driver\WriteConcern    成功返回对象，失败返回false
     */
    public function getWriteConcern($wait = null, $time = null, $safe = null) {
        if ($wait === null) $wait = \MongoDB\Driver\WriteConcern::MAJORITY;
        try {
            $this->err = array("code" => null, "msg" => null);
            return new \MongoDB\Driver\WriteConcern($wait, $time, $safe);
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }

    /**
     * 【插入一条数据】
     *
     * @param  string               $collection               集合名
     * @param  string|array|object  $document                 文档(对象/数组/JSON格式文本)
     * @param  int|string           $wait                     等待方式(w:1/0/majority)（默认值majority）
     * @param  int                  $time                     等待时间(ms)（此处默认值null，等于类初始化时的默认值）
     * @param  bool                 $safe                     等待写入journal（默认值null，根据系统设置）
     * @return bool|int                                       成功返回插入的ID，失败返回false
     */
    public function insertOne($collection = null, $document = null, $wait = null, $time = null, $safe = null) {
        if (is_array($document)) {

        } else if (is_object($document)) {
            $document = $this->_objToArray($document);
        } else if (is_string($document)) {
            $document = json_decode($document, true);
        } else {
            $this->err = array("code" => 999, "msg" => "document invalid");
            return false;
        }
        $document = array(0 => $document);
        $res = $this->insertMany($collection, $document, $wait, $time, $safe);
        if ($res === false) return false;
        return $res[0];
    }
    /**
     * 【插入一组数据】
     *
     * @param  string               $collection               集合名
     * @param  string|array|object  $document                 文档(对象/数组/JSON格式文本)
     * @param  int|string           $wait                     等待方式(w:1/0/majority)（默认值majority）
     * @param  int                  $time                     等待时间(ms)（此处默认值null，等于类初始化时的默认值）
     * @param  bool                 $safe                     等待写入journal（默认值null，根据系统设置）
     * @return bool|array                                     成功返回插入的ID数组，失败返回false
     */
    public function insertMany($collection = null, $document = null, $wait = null, $time = null, $safe = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($document)) {

        } else if (is_object($document)) {
            $document = $this->_objToArray($document);
        } else if (is_string($document)) {
            $document = json_decode($document, true);
        } else {
            $this->err = array("code" => 999, "msg" => "document invalid");
            return false;
        }

        $bulk = $this->getBulk();
        if ($bulk === false) return false;
        $write_concern = $this->getWriteConcern($wait, $time, $safe);
        if ($write_concern === false) return false;

        try {
            $ids = array();
            foreach ((array) $document as $val){
                if (!$val["_id"]) $val["_id"] = new \MongoDB\BSON\ObjectID;
                $ids[] = $bulk->insert($val);
            }
            $res = $this->_manager->executeBulkWrite($this->_db. '.' .$collection, $bulk, array("writeConcern" => $write_concern));
            // $c = $res->getInsertedCount();
            $this->err = array("code" => null, "msg" => null);
            return $this->_objToArray($ids, true);
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }

    /**
     * 【删除一条数据】
     *
     * @param  string               $collection       集合名
     * @param  string|array|object  $filter           条件(对象/数组/JSON格式文本)
     * @param  string|array|object  $option           参数(对象/数组/JSON格式文本)
     * @return bool|int|null                          成功返回删除数量，失败返回flase
     */
    public function deleteOne($collection = null, $filter = null, $option = null){
        return $this->deleteMany($collection, $filter, $option, true);
    }
    /**
     * 【删除一组数据】
     *
     * @param  string               $collection       集合名
     * @param  string|array|object  $filter           条件(对象/数组/JSON格式文本)
     * @param  string|array|object  $option           参数(对象/数组/JSON格式文本)
     * @param  bool                 $limit            只删除第一条匹配数据（默认值false）
     * @return bool|int|null                          成功返回删除数量，失败返回flase
     */
    public function deleteMany($collection = null, $filter = null, $option = null, $limit = false){
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        if ($option === null) {

        } else if (is_array($option)) {
            $option = array("collation" => $option);
        } else if (is_object($option)) {
            $option = array("collation" => $option);
        } else if (is_string($option)) {
            $option = array("collation" => json_decode($option, true));
        } else {
            $this->err = array("code" => 999, "msg" => "option invalid");
            return false;
        }
        if ($limit) $option["limit"] = true;

        $bulk = $this->getBulk();
        if ($bulk === false) return false;

        try {
            $bulk->delete($filter, $option);
            $res = $this->_manager->executeBulkWrite($this->_db. '.' .$collection, $bulk);
            $c = $res->getDeletedCount();
            $this->err = array("code" => null, "msg" => null);
            return $c;
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }

    /**
     * 【更新数据】
     *
     * @param  string               $collection   集合
     * @param  string|array|object  $filter       条件(对象/数组/JSON格式文本)
     * @param  string|array|object  $document     新值(对象/数组/JSON格式文本)
     * @param  bool                 $upsert       不存在是否新增(默认false)
     * @param  bool                 $multi        是否全部更新(默认false)，替换模式不能为true
     * @param  int|string           $wait         等待方式(w:1/0/majority)（默认值majority）
     * @param  int                  $time         等待时间(ms)（此处默认值null，等于类初始化时的默认值）
     * @param  bool                 $safe         等待写入journal（默认值null，根据系统设置）
     * @return bool|int                           成功返回受影响的数量，失败返回false
     */
    public function update($collection = null, $filter = null, $document = null, $upsert = null, $multi = null, $wait = null, $time = null, $safe = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        if (is_array($document)) {

        } else if (is_object($document)) {

        } else if (is_string($document)) {
            $document = json_decode($document, true);
        } else {
            $this->err = array("code" => 999, "msg" => "field invalid");
            return false;
        }
        if ($upsert !== null) $option["upsert"] = (bool) $upsert;
        if ($multi  !== null) $option["multi"]  = (bool) $multi;

        $bulk = $this->getBulk();
        if ($bulk === false) return false;
        $write_concern = $this->getWriteConcern($wait, $time, $safe);
        if ($write_concern === false) return false;

        try {
            $bulk->update($filter, $document, $option);
            $res = $this->_manager->executeBulkWrite($this->_db. '.' .$collection, $bulk, array("writeConcern" => $write_concern));
            $c = $res->getModifiedCount();
            if ($upsert) $c += $res->getUpsertedCount();
            $this->err = array("code" => null, "msg" => null);
            return $c;
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }

    /**
     * 【查询指定ID】
     *
     * @param  string              $collection      集合名
     * @param  int|string          $id              ID
     * @param  bool                $isObject        ID转为OBJ（默认值false）
     * @param  string|array|object $projection      返回数据格式
     * @return bool|array                           成功返回结果数组，失败返回false
     * @throws
     */
    public function selectById($collection = null, $id = null, $isObject = false, $projection = null){
        try {
            if ($isObject) $id = new \MongoDB\BSON\ObjectID($id);
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
        $filter = ['_id' => $id];
        return $this->query($collection, $filter, $projection);
    }
    /**
     * 【通用查询】
     *
     * @param  string              $collection      集合名
     * @param  string|array|object $filter          条件(对象/数组/JSON格式文本)  https://docs.mongodb.com/manual/tutorial/query-documents/  https://docs.mongodb.com/manual/core/document/#document-query-filter
     * @param  string|array|object $projection      返回数据格式
     * @param  string|array|object $sort            排序方式
     * @param  bool                $collation_zh    中文按拼音排序（默认值false）
     * @param  int                 $limit           返回结果数量
     * @param  int                 $skip            跳过文档数量（执行顺序：sort->skip->limit）
     * @return bool|array                           成功返回结果数组，失败返回false
     * @throws
     */
    public function query($collection = null, $filter = null, $projection = null, $sort = null, $collation_zh = false, $limit = null, $skip = null){
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        if ($projection === null) {

        } else if (is_array($projection)) {

        } else if (is_object($projection)) {

        } else if (is_string($projection)) {
            $projection = json_decode($projection, true);
        } else {
            $this->err = array("code" => 999, "msg" => "projection invalid");
            return false;
        }
        if ($sort === null) {

        } else if (is_array($sort)) {

        } else if (is_object($sort)) {

        } else if (is_string($sort)) {
            $sort = json_decode($sort, true);
        } else {
            $this->err = array("code" => 999, "msg" => "sort invalid");
            return false;
        }
        if ($limit !== null && !is_int($limit)) {
            $this->err = array("code" => 999, "msg" => "limit invalid");
            return false;
        }
        if ($skip !== null && !is_int($skip)) {
            $this->err = array("code" => 999, "msg" => "skip invalid");
            return false;
        }
        if ($projection)   $option["projection"] = $projection;
        if ($sort)         $option["sort"]       = $sort;
        if ($limit)        $option["limit"]      = (int) $limit;
        if ($skip)         $option["skip"]       = (int) $skip;
        if ($collation_zh) $option["collation"]  = array("locale" => "zh");

        try {
            $query = new \MongoDB\Driver\Query($filter, $option);
            $res = $this->_manager->executeQuery($this->_db. '.' .$collection, $query);
            $data = array();
            foreach ($res as $item) $data[] = $this->_objToArray($item, true);
            return $data;
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }

    /**
     * 【运行命令】
     *
     * https://docs.mongodb.com/master/reference/command/
     *
     * @param  string|array|object $cmd      命令
     * @return bool|array                    失败返回false，成功返回\MongoDB\Driver\Cursor对象转换后的数组
     * @throws
     */
    public function command($cmd = null) {
        if (is_array($cmd)) {

        } else if (is_object($cmd)) {
            $cmd = $this->_objToArray($cmd);
        } else if (is_string($cmd)) {
            $cmd = json_decode($cmd, true);
        } else {
            $this->err = array("code" => 999, "msg" => "command invalid");
            return false;
        }

        try{
            $cmd = new \MongoDB\Driver\Command($cmd);
            $res = $this->_manager->executeCommand($this->_db, $cmd)->toArray();
            $res = $this->_objToArray($res, true);
            return $res;
        }
        catch (\Exception $e) {
            $this->err = array("code" => $e->getCode(), "msg" => $e->getMessage());
            return false;
        }
    }
    /**
     * 【统计数量】
     *
     * @param  string              $collection 集合名
     * @param  array|object|string $filter     条件(数组/对象/JSON文本)
     * @return bool|int                        成功返回数量，失败返回false
     */
    public function count($collection = null, $filter = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if ($filter === null) {

        } else if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        $cmd = [
            'count' => $collection,
            'query' => $filter
        ];

        $res = $this->command($cmd);
        if ($res === false) return false;
        if (!$res[0] || !$res[0]["ok"] || !key_exists("n", $res[0])) {
            $this->err = array("code" => 999, "msg" => "resault invalid");
            return false;
        }
        return $res[0]["n"];
    }
    /**
     * 【聚合查询】
     *
     * @param  string              $collection  集合名
     * @param  array|object|string $pipeline    pipeline
     * @return bool|array                       失败返回false，成功返回结果数组
     */
    function aggregate($collection = null, $pipeline = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($pipeline)) {

        } else if (is_object($pipeline)) {
            $pipeline = $this->_objToArray($pipeline);
        } else if (is_string($pipeline)) {
            $pipeline = json_decode($pipeline, true);
        } else {
            $this->err = array("code" => 999, "msg" => "pipeline invalid");
            return false;
        }
        $pipeline = $this->_convFilter($pipeline);
        $cmd = [
            'aggregate' => $collection,
            'pipeline'  => $pipeline,
            'cursor'    => [ "batchSize" => 0 ]     // Mongo4.0要求必须提供此参数
        ];

        return $this->command($cmd);
    }
    /**
     * 【唯一查询】 distinct
     *
     * @param  string               $collection     集合名
     * @param  string               $key            查询的键
     * @param  array|object|string  $filter         条件(数组/对象/JSON文本)
     * @return bool
     */
    function distinct($collection = null, $key = null, $filter = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (!is_string($key) || $key === "") {
            $this->err = array("code" => 999, "msg" => "key invalid");
            return false;
        }
        if ($filter === null) {

        } else if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        $cmd = [
            'distinct' => $collection,
            'key'      => $key
        ];
        if ($filter) $cmd["query"] = $filter;

        $res = $this->command($cmd);
        if ($res === false) return false;
        if (!$res[0] || !$res[0]["ok"] || !key_exists("values", $res[0])) {
            $this->err = array("code" => 999, "msg" => "resault invalid");
            return false;
        }
        return $res[0]["values"];
    }
    /**
     * 【查找并修改】
     *
     * ！注意！无论过滤选项匹配出多少数据，只修改第一条
     *
     * @param  string              $collection      文档名
     * @param  array|object|string $document        新值(数组/对象/JSON文本)
     * @param  array|object|string $filter          条件(数组/对象/JSON文本)
     * @param  string|array|object $sort            排序方式
     * @param  bool                $collation_zh    中文按拼音排序（默认值false）
     * @param  bool                $upsert          不存在是否新增（默认false）
     * @param  bool                $return_new      返回更新后的内容（默认false）
     * @param  int|string          $wait            等待方式(w:1/0/majority)（默认值majority）
     * @param  int                 $time            等待时间(ms)（此处默认值null，等于类初始化时的默认值）
     * @param  bool                $safe            等待写入journal（默认值null，根据系统设置）
     * @return bool|mixed
     */
    function findAndModify($collection = null, $document = null, $filter = null, $sort = null, $collation_zh = false, $upsert = false, $return_new = false, $wait = null, $time = null, $safe = null) {
        if (!is_string($collection) || $collection === "") {
            $this->err = array("code" => 999, "msg" => "collection invalid");
            return false;
        }
        if (is_array($document)) {

        } else if (is_object($document)) {
            $document = $this->_objToArray($document);
        } else if (is_string($document)) {
            $document = json_decode($document, true);
        } else {
            $this->err = array("code" => 999, "msg" => "document invalid");
            return false;
        }
        if ($filter === null) {

        } else if (is_array($filter)) {

        } else if (is_object($filter)) {
            $filter = $this->_objToArray($filter);
        } else if (is_string($filter)) {
            $filter = json_decode($filter, true);
        } else {
            $this->err = array("code" => 999, "msg" => "filter invalid");
            return false;
        }
        if ($filter) $filter = $this->_convFilter($filter);
        if ($sort === null) {

        } else if (is_array($sort)) {

        } else if (is_object($sort)) {

        } else if (is_string($sort)) {
            $sort = json_decode($sort, true);
        } else {
            $this->err = array("code" => 999, "msg" => "sort invalid");
            return false;
        }

        $write_concern = $this->getWriteConcern($wait, $time, $safe);
        if ($write_concern === false) return false;

        $cmd = [
            'findAndModify' => $collection,
            'update'        => $document,
            'writeConcern'  => $write_concern
        ];
        if ($filter)       $cmd["query"]     = $filter;
        if ($sort)         $cmd["sort"]      = $sort;
        if ($collation_zh) $cmd["collation"] = array("locale" => "zh");
        if ($upsert)       $cmd["upsert"]    = (bool) $upsert;
        if ($return_new)   $cmd["new"]       = (bool) $return_new;

        $res = $this->command($cmd);
        if ($res === false) return false;
        if (!$res[0] || !$res[0]["ok"] || !key_exists("value", $res[0])) {
            $this->err = array("code" => 999, "msg" => "resault invalid");
            return false;
        }
        return $res[0]["value"];
    }

    /**
     * 【处理过滤器】
     *
     * 转换<>=等符号，解析正则表达式。
     * 正则语法与PHP相同 i.不区分大小写 m.^$以行模式匹配 s.允许.匹配换行符 x.忽略空格和注释
     *
     * @param  array $arr 要转换的数组
     * @return array      返回转换后的数组
     */
    public function _convFilter($arr) {
        $new = array();
        foreach ($arr as $k => $v)  {
            if (is_string($k)) {
                switch ($k) {
                    case ">"  : $k = "\$gt" ; break;
                    case ">=" : $k = "\$gte"; break;
                    case "<"  : $k = "\$lt" ; break;
                    case "<=" : $k = "\$lte"; break;
                    case "!=" : $k = "\$ne" ; break;
                    case "="  : $k = "\$eq" ; break;
                }
            }
            if (is_array($v)) {
                $new[$k] = $this->_convFilter($v);
            } else {
                if (preg_match("/^\/(.*)\/([imxs]*)$/", $v, $res)) {
                    $new[$k] = new \MongoDB\BSON\Regex($res[1], $res[2]);
                } else {
                    $new[$k] = $v;
                }
            }
        }
        return $new;
    }
    /**
     * 【对象转数组】
     *
     * @param  object $data         要转换的对象
     * @param  bool   $conv_id      处理<_id>（默认false）
     * @return array                返回生成的数组
     */
    public function _objToArray($data, $conv_id = false) {
        $data = json_decode(json_encode($data), true);
        if ($conv_id) $data = $this->_convId($data);
        return $data;
    }
    /**
     * 【处理数组中的ObjID】
     *
     * @param  array $arr 要转换的数组
     * @return array      返回转换后的数组
     */
    public function _convId($arr) {
        $new = array();
        foreach ($arr as $k => $v)  {
            if (is_array($v) && key_exists("\$oid", $v) && count($v) == 1) {
                $new[$k] = $v["\$oid"];
            } else if (is_array($v)) {
                $new[$k] = $this->_convId($v);
            } else {
                $new[$k] = $v;
            }
        }
        return $new;
    }
}
